/*
I Design Pattern Singleton assicura la creazione e l'utilizzo di un'unica istanza
all'interno di un programma.
Utile quando una classe è utilizzata da più thread e serve un'istanza unica.

Cavillo:
tale DP ci permette volendo di utilizzare la classe solo ed esclusivamente attraverso 
una sua interfaccia (non è necessario istanziarla), permettendo in un futuro di 
sostituirla in tronco senza problemi se necessario.
*/
public class Singleton{
	/*
	Dovrà essere possibile, in qualche modo, salvare l'istanza di tale
	classe indipendentemente dall'istanza.
	Per fare ciò salviamo come campo statico l'istanza di tale classe. 
	*/
	private static Singleton instance;

	/*
	nascondo il costruttore perchè non voglio che sia possibile
	poter creare una istanza di Singleton dall'esterno.
	Solamente quando getInstance lo riterrà nesessario potrà utilizarlo.
	Dall'esterno non sarà possibile fare new Singleton()
	*/ 
	private Singleton(){
		//ho creato una nuova istanza e quindi la salvo nel campo statico instance
		instance=this;
	}

	/*
	Tale metodo ritorna il un oggetto di tipo Singleton che è l'unica istanza
	di tale classe. Ciò ci è garantito dal fatto che viene controllato il valore
	del riferimento instance:
	* se non è null vuol dire chè è già stata creata un'istanza per tale classe
		e quindi ritorno essa;
	* se è null viene creata una nuova istanza e ritornata (instance==null => non
	esiste un'altra istanza di tale classe)
	*/
	public static Singleton getInstance(){
		if(instance!=null)
			return instance;
		return new Singleton();
	}

}